@@ -1,8 +1,5 @@
 /******************************************************************************/
 
-// Enable C++11 via this plugin (Rcpp 0.10.3 or later)
-// [[Rcpp::plugins(cpp11)]]
-
 #include <bigsparser/SFBM.h>
 
 using namespace Rcpp;
@@ -10,29 +7,40 @@ using namespace Rcpp;
 /******************************************************************************/
 
 // [[Rcpp::export]]
-NumericVector ld_scores_sfbm(Rcpp::Environment X, bool compact, int ncores) {
+NumericVector ld_scores_sfbm(Rcpp::Environment X,
+                             const IntegerVector& ind_sub,
+                             int ncores) {
 
   Rcpp::XPtr<SFBM> sfbm = X["address"];
   const NumericVector p = X["p"];
-  int m = sfbm->ncol();
   const double * data = sfbm->i_x();
 
+  int m2 = sfbm->ncol();
+  std::vector<bool> use(m2, false);
+  for (const int& k : ind_sub) use[k] = true;
+
+  int m = ind_sub.size();
   std::vector<double> res(m);
 
   int chunk_size = ceil(m / (10.0 * ncores));
 
-  if (compact) {
+  if (sfbm->is_compact()) {
+
+    std::vector<int> first_i = X["first_i"];
 
     #pragma omp parallel for schedule(dynamic, chunk_size) num_threads(ncores)
     for (int j = 0; j < m; j++) {
 
-      size_t lo = p[j];
-      size_t up = p[j + 1];
+      int j2 = ind_sub[j];
+      size_t lo = p[j2];
+      size_t up = p[j2 + 1];
+      int i = first_i[j2];
 
       double ld_j = 0;
 
-      for (size_t k = lo; k != up; k++)
-        ld_j += data[k] * data[k];
+      for (size_t k = lo; k != up; k++, i++)
+        if (use[i])
+          ld_j += data[k] * data[k];
 
       res[j] = ld_j;
     }
@@ -42,13 +50,15 @@ NumericVector ld_scores_sfbm(Rcpp::Environment X, bool compact, int ncores) {
     #pragma omp parallel for schedule(dynamic, chunk_size) num_threads(ncores)
     for (int j = 0; j < m; j++) {
 
-      size_t lo = 2 * p[j];
-      size_t up = 2 * p[j + 1];
+      int j2 = ind_sub[j];
+      size_t lo = 2 * p[j2];
+      size_t up = 2 * p[j2 + 1];
 
       double ld_j = 0;
 
       for (size_t k = lo + 1; k < up; k += 2)
-        ld_j += data[k] * data[k];
+        if (use[data[k - 1]])
+          ld_j += data[k] * data[k];
 
       res[j] = ld_j;
     }
